/* 
 * File:   main.c
 * Author: th
 * Display routine for KS0066U Display
 * PIC 16F726
 * Version 003
 */


#include <xc.h>
#include <stdint.h>
#include <string.h>

#include "def.h"
#include "lcd_hd44780_pic16.h"

static char row1[21];
static char row2[21];
static uint8_t pushrow1=0;
static uint8_t pushrow2=0;

char godi80version[20] = "GODI80B Version 003";

/*
 interrupt routine
 we check for change of LD1,LD2 and Reset here
 */
gtb80b_portb_t myPORTB,oldPORTB,xorbyte;

void interrupt isr(void) {
            
    uint8_t data;
    static uint8_t row1_counter=0;
    static uint8_t row2_counter=0;
    static uint8_t next_LD1_is_ctrl = 0;
    static uint8_t next_LD2_is_ctrl = 0;
    
    // Was it the port B interrupt on change?
    if (INTCONbits.RBIF) {
                        
        // read of the port, as per datasheet
        myPORTB.byte = PORTB;
        
        //check for reset
        if (myPORTB.bitv.RESET == 0)
        {            
            LCDWriteStringXY(0,0,"We have a           ");
            LCDWriteStringXY(0,1,"RESET condition     ");            
        }
        else 
        {
        
        //read the data
        data = PORTC;
        
        //do xor to see what has changed
        xorbyte.byte = myPORTB.byte ^ oldPORTB.byte;        
        
        //LD1?
        if ((xorbyte.bitv.LD1 == 1) && (myPORTB.bitv.LD1 == 1))         
        {         
            if ( data == 1 )
            {
                next_LD1_is_ctrl = 1;
            }
            else
            {                
                if (next_LD1_is_ctrl)
                {
                    //we only look for: set pointer to zero
                    if (data == 0xc0)               
                    {
                        pushrow1=1;
                        row1_counter = 0;
                    }
                }    
                else
                {
                    //data mode, we ignore MSB at the moment
                    data = data & 0x7f;
                    if ( row1_counter < 20 )
                    {
                    row1[row1_counter] = data;                
                    row1_counter++;
                    }
                }                
                //reset ctrl flag
                next_LD1_is_ctrl = 0;               
           }
        }//LD1                               

        //LD2?
        if ((xorbyte.bitv.LD2 == 1) && (myPORTB.bitv.LD2 == 1))         
        {         
            if ( data == 1 )
            {
                next_LD2_is_ctrl = 1;
            }
            else
            {                
                if (next_LD2_is_ctrl)
                {
                    //we only look for: set pointer to zero
                    if (data == 0xc0)               
                    {
                        pushrow2=1;
                        row2_counter = 0;
                    }
                }    
                else
                {
                    //data mode, we ignore MSB at the moment
                    data = data & 0x7f;
                    if ( row2_counter < 20 )
                    {
                    row2[row2_counter] = data;                
                    row2_counter++;
                    }
                }                
                //reset ctrl flag
                next_LD2_is_ctrl = 0;               
           }
        }//LD1                               
                
        
        //remember old setting
        oldPORTB.byte = myPORTB.byte;        
        
        } //not a reset condition
        
      // Reset the interrupt flag
      INTCONbits.RBIF = 0;
        
    } //RBIF
}//interrupt

/*
 * Main
 */
void main(void) {

//old lines, update display only if necessary
char oldrow1[21] = "";
char oldrow2[21] = "";
    
//Clock Source and Value: Internal 16MHz
OSCCONbits.IRCF0 = 1; 
OSCCONbits.IRCF1 = 1;

/* Port init */
// Port A
ANSELA = 0;   //All I/O pins are configured as digital
ADCON1=0x6;        // PortA Digital
TRISA = 0; // PORTA AllOutput Leds; DB4..DB7 & RW

// Port B
ANSELB = 0;   //All I/O pins are configured as digital
TRISB = 0b11111100;       // Port B0&1 Output (RS&E))
OPTION_REGbits.nRBPU = 0; // enable weak pull ups (global)
WPUB= 0b11100000;    //turn on weak pull up on RB7,6,5
                    //  B2..B3 NC (Input))
                    //  B5..B7 Input ( RESET ; LD1 ; LD2 ))
//Interrupts
IOCB = 0b11100000;     // interrupt on change enable for RB7, 6 & 5
INTCONbits.RBIE = 1; //enable interrupt on change
INTCONbits.GIE = 1; // Enable Global Interrupt

//Port C
TRISC = 0b11111111; // PORTC all Input
//Port E
TRISE = 0; //Input

//Init vars & others
oldPORTB.byte = 0;
Green_LED = 0;
Yellow_LED =0;
//we need to have null terminated string
row1[20] = row2[20] = '\0';

//Initialize the LCD Module
  LCDInit(LS_NONE);

  //Clear the display
  LCDClear();

  //Write a string
  LCDWriteStringXY(0,0,godi80version);
  LCDWriteStringXY(0,1,"12345678901234567890");

//now enable ints
ei();  

  while(1)
  {
      if (pushrow1)
      {
         //did it changed?
         if ( strcmp(row1, oldrow1) != 0) 
         {
         //let the yellow LED blink
         Yellow_LED = ~Yellow_LED;        
         LCDWriteStringXY(0,0,row1);
         //remember new setting
         strcpy(oldrow1, row1);
         }
         //reset the flag
         pushrow1=0;
      }
      
      if (pushrow2)
      {   
         //did it changed?
         if ( strcmp(row2, oldrow2) != 0) 
         {
         //let the Green LED blink
         Green_LED = ~Green_LED;
         LCDWriteStringXY(0,1,row2);
         //remember new setting
         strcpy(oldrow2, row2);
         }
         //reset the flag
         pushrow2=0;
      }
          
  }

} //main

